/* xsecure 2.0: secure a SUN console under X */

/*	Walter Milliken			(milliken@bbn.com)
 *	BBN Advanced Computers, Inc.
 *	10 Fawcett St.
 *	Cambridge, MA 02238
 */

/*	Christopher Lott	(cml@cis.ohio-state.edu)
 *	Modified to accept root's passwd as well
 *	3 November 1987					
 * 	Modified further 28 June 1988:
 *         -b option == (b)lack screen (default)
 *	   -c option == leave screen (c)lear - don't black it out
 *
 *      Modified 26 August 1988
 *         -r option == accept root's passwd; don't otherwise
 *         don't accept a passwd on the command line (security hole)
 */

/* Luis Fernandes (elf@ee.ryerson.ca)
 *
 * $Id: xsecure.c,v 1.2 1994/10/15 17:33:15 elf Exp $
 *
 * Modified to run under Xt under X11R4; the old version didn't compile AT ALL
 * June     1992 Begun hacking it
 * March 01 1993 Completed (hmmm, a 9 month gestation period, oh baby!)
 * March 10 1993 Eric Bina's (ebina@void.ncsa.uiuc.edu) fix-ups received 
 * March 16 1993 errno validation, clean-up & linting
 * Oct   15 1994 updated for R6
 *
 * Wanted to use it with xfishtank rather deperately, so neither -b nor 
 * -r options implemented (yet!).
 *
 * "...if all you told was turned to gold/if all you dreamed were new..."
 *                                                    -enya
 * 
 * Release 2.1 Copyright 1994, Luis Fernandes
 * 
 * Permission to use, copy, hack, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and 
 * that both that copyright notice and this permission notice appear in 
 * supporting documentation.
 *
 * This application is presented as is without any implied or written 
 * warranty.
 *
 */


#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Shell.h>
#include <X11/Xlib.h>

#include <stdio.h>
#include <ctype.h>
#include <strings.h>
#include <pwd.h>		 		/* to use password functions*/

#include <sys/errno.h>			/* errono macros */
#include <signal.h>				/* to get SIGUSR1 */

#define LENGTH		16

char *crypt();

#if !defined(AIXV3) & !defined(sgi) & !defined(DEC) & !defined(hpux)
char *sprintf();
#endif

static int saverinterval, saverblank, saverexpose, savertimeout;

Display *display;
int screen;

XtAppContext app_con;
Widget topLevel, locker;
Window w;	/* translate widget to windowId*/

int xfishpid;


void
evalkillerror()
{
  extern errno;
  
  switch(errno)
	{
	case EINVAL:
	  /*unlikely*/
	  (void)fprintf(stderr,"xsecure: signal-number is not valid.\n");
	  break;
	  
	case EPERM:
	  /* if and when we get the root-pw option going*/
	  (void)fprintf(stderr,"xsecure: effective id not root!.\n");
	  break;
	  
	  case ESRCH:
	  (void)fprintf(stderr,"xsecure: %d is not a valid process-id.\n", 
					xfishpid);
	  break;
	}

  (void)fprintf(stderr,"Aborting!\n");
  
  exit(1);
  
}

/* Read in what user types-in, validate input when user types CR otherwise
 * continue storing input.
 */
XtEventHandler
/*ARGSUSED*/
HandleKeyboardInput(w, ppid, ev)
Widget w;
XtPointer ppid;
XEvent *ev;
{
  int pid=(int)ppid;
  
  struct passwd *pw;
  static int bp;
  static char buffer[LENGTH];
  static char user_pass[LENGTH*2];
  static char u[LENGTH*2];			/* space to encrypt what user enters */

  char c;
  
  /* only handle KeyPress event */
  switch(ev->xany.type)
	{
	case KeyPress:
	  XLookupString((XKeyEvent *)ev, &c, 1, NULL, NULL);

	  if (c == '\r')
		{
		  buffer[bp]='\0';
		  bp=0;	/*point to beginning, for next trial*/
		  
		  pw = getpwuid(getuid());		/* get user's system pw  */
		  strcpy(u, crypt(buffer, pw->pw_passwd)); 
		  
		  if(!strcmp(u, pw->pw_passwd))	/* not a password hacker */
			{
			  if(pid)
				{
				  /* restore screen saver*/
				  XSetScreenSaver(display,
								  savertimeout, saverinterval, 
								  saverblank, saverexpose);
				  
				  /*send SIGUSR1 to specified process*/
				  if(kill(pid, SIGUSR1)) evalkillerror();
				}
			  
			  XtDestroyApplicationContext(app_con);
			  exit(0);			/* away w' ye */
			}
		}
	  else
		{
		  buffer[bp]=c;
		  if (bp<LENGTH-1) bp++;	/* don't go past buffer[]*/
		}
	  break;
	  
	default:
	  /*do nothing*/
	  break;
	  
	} /* switch */
  
}

void
main(argc, argv)
int argc;
char * argv[];
{
  unsigned int width, height;
  
  topLevel=XtAppInitialize(&app_con, "XSecure", NULL, 0, &argc, argv,
						   NULL , NULL, 0);
  
  if(argc==2)					/* assume argv is pid of xfish */
	xfishpid=atoi(argv[1]);
  
  display=XtDisplay(topLevel);
  screen = DefaultScreen(display);
  
  width=height=1;				/* always in clear-mode */
  
  locker=XtVaAppCreateShell("xsecure", "XSecure", 
							overrideShellWidgetClass, display,
							XtNx, -1,
							XtNy, -1,
							XtNwidth, width,
							XtNheight, height,
							NULL);
  
  XtAddEventHandler(locker, (Cardinal) KeyPressMask, NULL,
					(XtEventHandler)HandleKeyboardInput, 
					(XtPointer)xfishpid);
  
  XtRealizeWidget(locker);
  w=XtWindow(locker);
  
  /* set the focus state to our window; when app. finishes, focus will revert
   * to Root-window*/
  XSetInputFocus(display, w, RevertToPointerRoot, CurrentTime);
  XGrabPointer(display, w, True, NULL, GrabModeAsync, GrabModeAsync, 
			   None, None, CurrentTime);
  
  /*send SIGUSR1 to process*/
  if(argc==2)
	{
	  XGetScreenSaver(display, &savertimeout, &saverinterval, &saverblank, 
					  &saverexpose);
	  XSetScreenSaver(display, 0, 0, 0, 0);	/* disable screen saver */
	  
	  if(kill(xfishpid, SIGUSR1)) evalkillerror();
	}
  
  XtAppMainLoop(app_con);
  
}
